/*
 * Program: SSL-WOPR Server
 * Source : wopr.h
 * Purpose: Declaration of WOPR_server class.
 *
 * Copyright (c) 2009, 2010 Oliver Mueller.
 * All rights reserved.
 * http://www.cogito-ergo-sum.org
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the author be held liable for any damages arising from
 * the use of this software.
 *
 */

#ifndef __WOPR_H
#define __WOPR_H

#include <iostream>
#include <iomanip>

#include <openssl/bio.h>
#include <openssl/ssl.h>

using namespace std;

#define STA_LOGSET    0x01
#define STA_LOGUNS    0x02
#define STA_LGNOLY    0x04
#define STA_LGFOLY    0x08
#define STA_SHTDWN    0x10

typedef struct
{
    const char *cmd;
    unsigned cmd_len;
    const char *resp;
    unsigned resp_len;
    short status;
} CmdNResp;

class SSL_exception;
class Overflow_exception;

class WOPR_server
{
public:
    WOPR_server(char *host, const char *cert, const char *key) throw (SSL_exception);
    ~WOPR_server();
    void run() throw (SSL_exception);

protected:
    SSL_CTX *ctx;
    SSL *ssl;
    BIO *abio, *client;
    unsigned long clno;
    char errmsgbuf[ERRSIZE];

    static bool ssl_initialized;
    static bool wopr_initialized;

    static const char *identification;
    static const char *prompt[2];
    static unsigned prompt_len[2];
    static const char *unknown_cmd;
    static unsigned unknown_cmd_len;
    static const char *unknown_user;
    static unsigned unknown_user_len;
    static CmdNResp cmd_n_resp[];

    static const char *access_denied;
    static const char *already_running;

    void handle_client() throw (Overflow_exception);
    void print_client_error(const char *s);
    void print_client_info(const char *s);
    void trim_all(char *s);
};

inline void WOPR_server::print_client_error(const char *s)
{
    cerr << "(" << right << setw(4) << setfill('0') << clno;
    cerr << "): " << s << "." << endl;
}

inline void WOPR_server::print_client_info(const char *s)
{
    cout << "(" << right << setw(4) << setfill('0') << clno;
    cout << "): " << s << "." << endl;
}

#endif
